﻿using System;
using System.Windows;
using System.Windows.Data;
using System.Windows.Threading;

namespace WpfApplication1
{
    internal static class FocusControl
    {
        #region FocusableProperty

        internal static DependencyProperty GetFocusableProperty(DependencyObject obj)
        {
            return (DependencyProperty)obj.GetValue(FocusablePropertyProperty);
        }

        internal static void SetFocusableProperty(DependencyObject obj, DependencyProperty value)
        {
            obj.SetValue(FocusablePropertyProperty, value);
        }

        internal static readonly DependencyProperty FocusablePropertyProperty =
            DependencyProperty.RegisterAttached(
            "FocusableProperty",
            typeof(DependencyProperty),
            typeof(FocusControl),
            new UIPropertyMetadata(null, OnFocusablePropertyChanged));

        static void OnFocusablePropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var focusController = sender.GetValue(FrameworkElement.DataContextProperty) as IFocusController;
            if (focusController == null)
            {
                var handler = sender.GetValue(FocusControlHandlerProperty) as FocusControlHandler;
                if (handler != null)
                {
                    handler.ReleaseReferences();
                    sender.ClearValue(FocusControlHandlerPropertyKey);
                }
            }
            else
            {
                var elem = sender as UIElement;
                var property = e.NewValue as DependencyProperty;
                var handler = new FocusControlHandler(elem, property);
                focusController.MoveFocus += handler.HandleMoveFocus;
                sender.SetValue(FocusControlHandlerPropertyKey, handler);
            }
        }

        #endregion // FocusableProperty

        #region FocusControlHelper

        static readonly DependencyPropertyKey FocusControlHandlerPropertyKey =
            DependencyProperty.RegisterAttachedReadOnly(
            "FocusControlHandler",
            typeof(FocusControlHandler),
            typeof(FocusControl),
            new UIPropertyMetadata(null));

        static readonly DependencyProperty FocusControlHandlerProperty = FocusControlHandlerPropertyKey.DependencyProperty;

        internal class FocusControlHandler
        {
            public FocusControlHandler(UIElement element, DependencyProperty property)
            {
                _element = element;
                _property = property;
            }

            internal void HandleMoveFocus(object sender, MoveFocusEventArgs e)
            {
                if (_element == null || _property == null)
                    return;

                var binding = BindingOperations.GetBinding(_element, _property);
                if (binding == null)
                    return;

                if (e.FocusedProperty != binding.Path.Path)
                    return;

                // Delay the call to allow the current batch
                // of processing to finish before we shift focus.
                _element.Dispatcher.BeginInvoke((Action)delegate
                {
                    _element.Focus();
                },
                DispatcherPriority.Background);
            }

            internal void ReleaseReferences()
            {
                _element = null;
                _property = null;
            }

            UIElement _element;
            DependencyProperty _property;
        }

        #endregion // FocusControlHelper
    }
}